@@ -21,7 +21,6 @@ a single value.
 
 Notably, the set of steps available at any point may depend on the
 execution to date.
-
 """
 
 
@@ -41,16 +40,14 @@ from hypothesis.control import BuildContext
 from hypothesis._settings import Verbosity
 from hypothesis._settings import settings as Settings
 from hypothesis.reporting import report, verbose_report, current_verbosity
-from hypothesis.strategies import just, lists, builds, one_of, runner, \
-    integers
+from hypothesis.strategies import just, one_of, runner, tuples, \
+    fixed_dictionaries
 from hypothesis.vendor.pretty import CUnicodeIO, RepresentationPrinter
 from hypothesis.internal.reflection import proxies, nicerepr
 from hypothesis.internal.conjecture.data import StopTest
 from hypothesis.internal.conjecture.utils import integer_range, \
     calc_label_from_name
 from hypothesis.searchstrategy.strategies import SearchStrategy
-from hypothesis.searchstrategy.collections import TupleStrategy, \
-    FixedKeysDictStrategy
 
 STATE_MACHINE_RUN_LABEL = calc_label_from_name('another state machine step')
 
@@ -102,7 +99,6 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
     state_machine_factory is anything which returns an instance of
     GenericStateMachine when called with no arguments - it can be a class or a
     function. settings will be used to control the execution of the test.
-
     """
     try:
         breaker = find_breaking_runner(state_machine_factory, settings)
@@ -138,7 +134,6 @@ class GenericStateMachine(object):
 
     And if this ever produces an error it will shrink it down to a small
     sequence of example choices demonstrating that.
-
     """
 
     def steps(self):
@@ -154,7 +149,6 @@ class GenericStateMachine(object):
         """Print a step to the current reporter.
 
         This is called right before a step is executed.
-
         """
         self.step_count = getattr(self, u'step_count', 0) + 1
         report(u'Step #%d: %s' % (self.step_count, nicerepr(step)))
@@ -164,7 +158,6 @@ class GenericStateMachine(object):
         state.
 
         Does nothing by default
-
         """
         pass
 
@@ -216,7 +209,6 @@ class StateMachineRunner(object):
     """A StateMachineRunner is a description of how to run a state machine.
 
     It contains values that it will use to shape the examples.
-
     """
 
     def __init__(self, data, n_steps):
@@ -229,25 +221,19 @@ class StateMachineRunner(object):
             print_steps = current_verbosity() >= Verbosity.debug
         self.data.hypothesis_runner = state_machine
 
-        stopping_value = 1 - 1.0 / (1 + self.n_steps * 0.5)
+        should_continue = cu.many(
+            self.data, min_size=1, max_size=self.n_steps,
+            average_size=self.n_steps,
+        )
+
         try:
             state_machine.check_invariants()
 
-            steps = 0
-            while True:
-                if steps >= self.n_steps:
-                    stopping_value = 0
-                self.data.start_example(STATE_MACHINE_RUN_LABEL)
-                if not cu.biased_coin(self.data, stopping_value):
-                    self.data.stop_example()
-                    break
-                assert steps < self.n_steps
+            while should_continue.more():
                 value = self.data.draw(state_machine.steps())
-                steps += 1
                 if print_steps:
                     state_machine.print_step(value)
                 state_machine.execute_step(value)
-                self.data.stop_example()
                 state_machine.check_invariants()
         finally:
             state_machine.teardown()
@@ -283,8 +269,8 @@ class Bundle(SearchStrategy):
         bundle = machine.bundle(self.name)
         if not bundle:
             data.mark_invalid()
-        reference = bundle.pop()
-        bundle.insert(integer_range(data, 0, len(bundle)), reference)
+        i = integer_range(data, 0, len(bundle) - 1)
+        reference = bundle[i]
         return machine.names_to_values[reference.name]
 
 
@@ -304,7 +290,6 @@ def rule(targets=(), target=None, **kwargs):
     invocation. If their value is a Bundle then values that have previously
     been produced for that bundle will be provided, if they are anything else
     it will be turned into a strategy and values from that will be provided.
-
     """
     if target is not None:
         targets += (target,)
@@ -359,7 +344,6 @@ def precondition(precond):
 
     This is better than using assume in your rule since more valid rules
     should be able to be run.
-
     """
     def decorator(f):
         @proxies(f)
@@ -403,7 +387,6 @@ def invariant():
             @invariant()
             def is_nonzero(self):
                 assert self.state != 0
-
     """
     def accept(f):
         existing_invariant = getattr(f, INVARIANT_MARKER, None)
@@ -424,12 +407,6 @@ def invariant():
     return accept
 
 
-@attr.s()
-class ShuffleBundle(object):
-    bundle = attr.ib()
-    swaps = attr.ib()
-
-
 class RuleBasedStateMachine(GenericStateMachine):
     """A RuleBasedStateMachine gives you a more structured way to define state
     machines.
@@ -439,7 +416,6 @@ class RuleBasedStateMachine(GenericStateMachine):
     from bundles (or just from normal strategies) and push data onto
     bundles. At any given point a random applicable rule will be
     executed.
-
     """
 
     _rules_per_class = {}
@@ -546,27 +522,17 @@ class RuleBasedStateMachine(GenericStateMachine):
                         break
                 converted_arguments[k] = v
             if valid:
-                strategies.append(TupleStrategy((
-                    just(rule),
-                    FixedKeysDictStrategy(converted_arguments)
-                ), tuple))
+                strategies.append(tuples(
+                    just(rule), fixed_dictionaries(converted_arguments)
+                ))
         if not strategies:
             raise InvalidDefinition(
                 u'No progress can be made from state %r' % (self,)
             )
 
-        for name, bundle in self.bundles.items():
-            if len(bundle) > 1:
-                strategies.append(
-                    builds(
-                        ShuffleBundle, just(name),
-                        lists(integers(0, len(bundle) - 1))))
-
         return one_of(strategies)
 
     def print_step(self, step):
-        if isinstance(step, ShuffleBundle):
-            return
         rule, data = step
         data_repr = {}
         for k, v in data.items():
@@ -580,11 +546,6 @@ class RuleBasedStateMachine(GenericStateMachine):
         ))
 
     def execute_step(self, step):
-        if isinstance(step, ShuffleBundle):
-            bundle = self.bundle(step.bundle)
-            for i in step.swaps:
-                bundle.insert(i, bundle.pop())
-            return
         rule, data = step
         data = dict(data)
         result = rule.function(self, **data)
